@@ -1,17 +1,17 @@
 ;;;; build-gtk.jl -- translate guile-gtk .defs file to rep C code
 ;;;  Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
-;;;  $Id: build-gtk.jl,v 1.20 2002/02/13 07:50:27 jsh Exp $
+;;;  $Id$
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
 ;;; the Free Software Foundation; either version 2, or (at your option)
 ;;; any later version.
-;;; 
+;;;
 ;;; This program is distributed in the hope that it will be useful,
 ;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;;; GNU General Public License for more details.
-;;; 
+;;;
 ;;; You should have received a copy of the GNU General Public License
 ;;; along with this software; see the file COPYING.  If not, write to
 ;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -840,7 +840,7 @@
     (output-helper inner-type standard-output)
     (@ "sgtk_rep_to_%s \(%s, _sgtk_helper_fromrep_%s\)"
        outer-type rep-var inner-type)))
-       
+
 (defun output-list-to-rep (output type gtk-var typage)
   (declare (unused typage))
   (let
@@ -918,6 +918,8 @@
 		  (@ "  %s c_%s;\n" decl (gtk-arg-name arg))
 		(gtk-warning
 		 "Don't know how to declare type: %s" type)))) args)
+    (when (gtk-get-option 'gerror-arg options)
+      (@ "  GError* error = NULL;\n"))
     (unless (eq ret 'none)
       (let*
 	  ((typage (gtk-type-info ret))
@@ -950,7 +952,7 @@
 	  (setq tem (cdr tem))
 	  (setq i (1+ i)))
 	(@ "\n")))
-    
+
     ;; output arg checks and conversions
     (let
 	((tem args)
@@ -1018,7 +1020,7 @@
 	      (@ ";\n"))) args)
     (when args
       (@ "\n"))
-    
+
     (if function-callback
 	(funcall function-callback output)
       ;; output call
@@ -1040,6 +1042,8 @@
 	      (@ "c_%s" (gtk-arg-name (car tem)))))
 	  (@ (if (cdr tem) ", " ""))
 	  (setq tem (cdr tem))))
+      (if (gtk-get-option 'gerror-arg options)
+	  (@ ", &error"))
       (@ "\);\n\n"))
 
     ;; output ret conversion
@@ -1076,6 +1080,11 @@
 	      (@ "  rep_POPGC;\n"
 		 (gtk-arg-name arg) (gtk-arg-name arg)))) args)
 
+    ;; gerror checking
+    (when (gtk-get-option 'gerror-arg options)
+      (@ "  if (error != NULL)\n" )
+      (@ "    sgtk_throw_gerror (\"%s\", error);\n" fname))
+    
     ;; output return statement
     (if (eq ret 'none)
 	(@ "  return Qnil;\n")
@@ -1203,6 +1212,12 @@
   (let
       ((out nil)
        (point 0))
+
+    ;; Some Classes (GtkUIManager) contain Upcase Tokens: UI
+    (while (string-match "[A-Z]([A-Z]+)[A-Z]" name)
+      (let ((upcase-token (substring name (match-start 1) (match-end 1))))
+	(setq name (string-replace upcase-token (string-downcase upcase-token) name))))
+
     (while (string-match "[A-Z]+" name point)
       (setq out (cons (substring name point (match-start)) out))
       (unless (zerop point)
@@ -1237,6 +1252,12 @@
 (define-type 'type "GtkType" "sgtk_rep_to_type"
 	     "sgtk_type_to_rep" "sgtk_valid_type")
 
+(define-type 'GValue "GValue" "sgtk_rep_to_gvalue"
+             "sgtk_gvalue_to_rep" "sgtk_valid_gvalue")
+
+(define-type 'GtkArg "GtkArg" "sgtk_rep_to_arg"
+             "sgtk_arg_to_rep" "sgtk_valid_arg")
+
 (define-type 'char "gchar" "sgtk_rep_to_char"
 	     "sgtk_char_to_rep" "sgtk_valid_char")
 
@@ -1245,6 +1266,7 @@
 ;; XXX fix the validation functions
 (define-type 'short "short" "sgtk_rep_to_int" "sgtk_int_to_rep"
 	     "sgtk_valid_int" '(listable . t))
+
 (define-type 'ushort "gushort" "sgtk_rep_to_uint" "sgtk_uint_to_rep"
 	     "sgtk_valid_uint" '(listable . t))
 
@@ -1254,6 +1276,9 @@
 (define-type 'uint "guint" "sgtk_rep_to_uint" "sgtk_uint_to_rep"
 	     "sgtk_valid_uint" '(listable . t))
 
+(define-type 'GQuark "guint" "sgtk_rep_to_uint" "sgtk_uint_to_rep"
+	     "sgtk_valid_uint" '(listable . t))
+
 (define-type 'long "glong" "sgtk_rep_to_long"
 	     "sgtk_long_to_rep" "sgtk_valid_long")
 
@@ -1278,7 +1303,7 @@
 (define-type 'boxed output-complex-type output-rep-to-boxed
 	     output-boxed-to-rep output-boxed-pred '(listable . t))
 
-(define-type 'pointer "gpointer" "sgtk_rep_to_pointer"
+(define-type 'GPointer "gpointer" "sgtk_rep_to_pointer"
 	     "sgtk_pointer_to_rep" "sgtk_valid_pointer")
 
 (define-type 'object output-complex-type output-rep-to-object
@@ -1291,7 +1316,7 @@
 	     "sgtk_valid_function" (cons 'c2args output-full-callback-args)
 	     (cons 'finish output-full-callback-finish))
 
-(define-type 'gclosure "GClosure*" output-rep-to-gclosure nil
+(define-type 'GClosure "GClosure*" output-rep-to-gclosure nil
 	     "sgtk_valid_function" (cons 'finish output-full-callback-finish))
 
 (define-type 'file-descriptor "int" "sgtk_rep_to_fd"
@@ -1326,7 +1351,7 @@
 (define-type 'double "gdouble" "sgtk_rep_to_double"
 	     "sgtk_double_to_rep" "sgtk_valid_double")
 
-(define-type 'point "GdkPoint" "sgtk_rep_to_point"
+(define-type 'GdkPoint "GdkPoint" "sgtk_rep_to_point"
 	     "sgtk_point_to_rep" "sgtk_valid_point")
 
 (define-type 'rect "GdkRectangle" "sgtk_rep_to_rect"
