@@ -1,18 +1,18 @@
 /* Copyright (C) 1997, 1998, 1999 Marius Vollmer
  * Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
  *
- * $Id: rep-gtk.h 384 2008-10-18 18:07:11Z chrisb $
- * 
+ * $Id$
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
  * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -224,6 +224,8 @@ GParameter *sgtk_build_args (GObjectClass *objclass, int *n_argsp,
 repv sgtk_color_conversion (repv color);
 repv sgtk_font_conversion (repv color);
 
+void sgtk_throw_gerror (const char *func_name, GError *gerr);
+
 void sgtk_set_standalone (int flag);
 int sgtk_is_standalone (void);
 repv sgtk_standalone_p (void);
@@ -332,6 +334,7 @@ void gtk_signal_set_class_function_full (GtkType            type,
 
 void gtk_color_selection_set_color_interp (GtkColorSelection *sel, GdkColor *color);
 GdkColor *gtk_color_selection_get_color_interp (GtkColorSelection *sel);
+char *gtk_color_button_get_color_interp (GtkColorButton *button);
 extern void gtk_widget_draw_interp (GtkWidget *widget);
 
 GtkTextIter *gtk_text_iter_new (void);
